package Application;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.util.Observable;


public class CurrencyManager extends Observable implements Runnable {
	
	public CurrencyManager(){
	     
	}

	private TreeMap<String,Float> currencylist;

	public void updateCurrencies() throws ParserConfigurationException, SAXException, IOException, ClassNotFoundException {
		if (currencylist==null)
			currencylist = new TreeMap<String,Float>();
	
		String xmlfile = "";
	    long fileage= (long) 0;
	    long currenttime = System.currentTimeMillis();
	    File file = new File("curr.pobj");
		fileage = file.lastModified();

		
	    if (currenttime-fileage >=86400000){
	    	System.out.println("updating file\n");
	    	xmlfile = this.getURLContent("https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml");
		
	    	ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("curr.pobj"));
	    	oos.writeObject(xmlfile);
	    	oos.close();
	    }
	    else{
	    	System.out.println("using saved currency file, file is younger than 24h\n");
	    	ObjectInputStream ois = new ObjectInputStream(new FileInputStream("curr.pobj"));
			xmlfile = (String) ois.readObject(); // cast is needed.
			ois.close();
	    }
		
		
		
		DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
		
		ByteArrayInputStream stream = new ByteArrayInputStream(xmlfile.getBytes());
		Document doc = dBuilder.parse(stream);
		
		doc.getDocumentElement().normalize();	
		
		NodeList nList = doc.getElementsByTagName("Cube");
       
		for (int i=0; i< nList.getLength(); i++){
			Node nNode = nList.item(i);
			if (nNode.getNodeType() == Node.ELEMENT_NODE) {
				Element e = (Element) nNode;
				if(!e.getAttribute("currency").isEmpty())
					currencylist.put(e.getAttribute("currency"),Float.parseFloat(e.getAttribute("rate")));
			}
		}
	}
	
	public TreeMap<String,Float> getCurrencyList() {
		return currencylist;
	}
	
	public String getURLContent(String p_sURL)
	{
	    URL oURL;
	    URLConnection oConnection;
	    BufferedReader oReader;
	    String sLine;
	    StringBuilder sbResponse;
	    String sResponse = null;

	    try
	    {
	        oURL = new URL(p_sURL);
	        oConnection = oURL.openConnection();
	        oReader = new BufferedReader(new InputStreamReader(oConnection.getInputStream()));
	        sbResponse = new StringBuilder();

	        while((sLine = oReader.readLine()) != null)
	        {
	            sbResponse.append(sLine);
	        }

	        sResponse = sbResponse.toString();
	    }
	    catch(Exception e)
	    {
	        e.printStackTrace();
	    }

	    return sResponse;
	}

	@Override
	public void run() {
		// TODO Auto-generated method stub
		try {
			updateCurrencies();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println(this.toString());
		setChanged();
		this.notifyObservers();
	}

}